class AmariNeuron
{
  //activation variable u
  float u;
  float dot_u;
  float output;
  //affernt neuron
  ArrayList afferent_neurons;
  float [] afferent_weights;
  //parameter section
  //timescale \tau
  float tau;
  //resting level
  float h;
  //slope \beta of sigmoid function
  float beta;
  //self excitation
  float c_uu;
  //noise strength
  float c_n;

  //afferent connection strength
  float c_afferent;
  //drawing parameters
  int pos_x, pos_y;
  int gray_value;
  float t,d_t,o_t;
  
  //constructor
  AmariNeuron(float u_0, float tau, float h, float beta, float c_uu, float c_n, int pos_x, int pos_y, int gray_value)
  {
    this.tau=tau;
    this.h=h;
    this.pos_x=pos_x;
    this.pos_y=pos_y;
    this.gray_value=gray_value;
    this.beta=beta;
    this.c_uu=c_uu;
    this.c_n=c_n;
    u=u_0;
    output=sigmoid(u, beta);
    afferent_neurons=new ArrayList(); 
    afferent_weights=new float[0];
    randomSeed(0);
  }
  void setAfferentConnection(AmariNeuron afferentNeuron, float c_afferent)
  {
    this.afferent_neurons.add(afferentNeuron);
    this.afferent_weights=append(this.afferent_weights, c_afferent);
  }
  void computeOutput()
  {
    output=sigmoid(u, beta);
  }
  void compute()
  {
    t=millis();
    //measure time since last call of loop
    d_t=t- o_t;
    o_t=t;
    if (d_t>tau)
    {
      d_t=tau;
    }
    if (d_t==0)
    {
      d_t=1;
    }
    float s=0;
    for (int i=0;i<afferent_neurons.size();i++)
    {
      AmariNeuron afferent_neuron= (AmariNeuron) afferent_neurons.get(i);
      c_afferent= afferent_weights[i];
      s+=c_afferent*afferent_neuron.output;
    }
    float n=random(-1, 1);
    dot_u=d_t/tau * (-u  + c_uu*output + s + c_n*n + h);
    u+=dot_u;
  }


  void drawConnections()
  { 
    for (int i=0;i<afferent_neurons.size();i++)
    {   
      AmariNeuron afferent_neuron= (AmariNeuron) afferent_neurons.get(i);
      //line(afferent_neuron.pos_x, afferent_neuron.pos_y, pos_x, pos_y);

      int distance_x=(pos_x-afferent_neuron.pos_x);
      int distance_y=(pos_y-afferent_neuron.pos_y);
      float distance=sqrt(pow(distance_x, 2)+pow(distance_y, 2));
      float direction=atan2(distance_y, distance_x);
      noFill();
      strokeWeight(2*abs(afferent_weights[i]));  
      stroke((1-afferent_neuron.output)*200); 
      float intermediate_point_x1=afferent_neuron.pos_x+distance/3*cos(direction) -  distance/3*sin(direction);
      float intermediate_point_x2=afferent_neuron.pos_x+2*distance/3*cos(direction) -  distance/3*sin(direction);
      float intermediate_point_y1=afferent_neuron.pos_y+distance/3*sin(direction) + distance/3*cos(direction);
      float intermediate_point_y2=afferent_neuron.pos_y+2*distance/3*sin(direction) +  distance/3*cos(direction);

      bezier(afferent_neuron.pos_x, afferent_neuron.pos_y, intermediate_point_x1, intermediate_point_y1, intermediate_point_x2, intermediate_point_y2, pos_x, pos_y);
      float bezier_x = bezierPoint(afferent_neuron.pos_x, intermediate_point_x1, intermediate_point_x2, pos_x, .8);
      float bezier_y = bezierPoint(afferent_neuron.pos_y, intermediate_point_y1, intermediate_point_y2, pos_y, .8);
      // ellipse(bezier_x,bezier_y,5,5);
      fill((1-afferent_neuron.output)*200);
      if (afferent_weights[i]<0)
      {
        direction=atan2( pos_y-bezier_y, pos_x-bezier_x);
        //pushMatrix();  
        float elipse_x=pos_x-(10+30*output)/2*cos(direction);
        float elipse_y=pos_y-(10+30*output)/2*sin(direction);

        //   translate(elipse_x, elipse_y);
        // rotate(direction);
        ellipse(elipse_x, elipse_y, 10, 10);
        //popMatrix();
      }
      else
      {
        direction=atan2( pos_y-bezier_y, pos_x-bezier_x);
        pushMatrix();  
        float elipse_x=pos_x-(10+30*output)/2*cos(direction);
        float elipse_y=pos_y-(10+30*output)/2*sin(direction);

        translate(elipse_x, elipse_y);
        rotate(direction+PI/4);
        rectMode(CENTER);
        rect(0, 0, 8, 8);
        popMatrix();
      }
    }
    if (c_uu!=0)
    {
      stroke((1-output)*200);      
      fill((1-output)*200);
      float bezier_x = bezierPoint(pos_x, pos_x-50, pos_x-50, pos_x, .8);
      float bezier_y = bezierPoint(pos_y, pos_y+50, pos_y-50, pos_y, .8);
      float direction=atan2( pos_y-bezier_y, pos_x-bezier_x);
      pushMatrix();  
      float elipse_x=pos_x-(10+30*output)/2*cos(direction);
      float elipse_y=pos_y-(10+30*output)/2*sin(direction);

      translate(elipse_x, elipse_y);
      rotate(direction+PI/4);
      rectMode(CENTER);
      rect(0, 0, 8, 8);
      popMatrix();
      noFill();

      bezier(pos_x, pos_y, pos_x- 50, pos_y + 50, pos_x-50, pos_y-50, pos_x, pos_y);
    }
  }
  void drawActivation()
  {
    noStroke();
    fill(gray_value);
    ellipse(pos_x, pos_y, 15+25*output, 15+25*output);
  }
  float sigmoid(float x, float beta)
  {
    return 1./(1+exp(-beta*x));
  }
}

