/*
* coupled oscillators
 */

AmariNeuron u1, v1, u2, v2;
HScrollbar hs1;
int ctr;
void setup ()
{
  size(250, 250);
  background( 255 );
  u1 = new  AmariNeuron(0,300, -.0155, 14.905, 1.2, .4, 80, 100, 50);
  v1 = new  AmariNeuron(0.5,300, -1.    , 14.905, 0    , .4, 200, 100, 120);
  u1.setAfferentConnection(v1, -1.57);
  v1.setAfferentConnection(u1, 1.57);
  u2 = new  AmariNeuron( 0.5, 300, -.0155, 14.905, 1.2, .4, 80, 200, 50);
  v2 = new  AmariNeuron(0, 300, -1.    , 14.905, 0    , .4, 200, 200, 120);
  u2.setAfferentConnection(v2, -1.57);
  v2.setAfferentConnection(u2, 1.57);
  v2.setAfferentConnection(u1, 1);
  v1.setAfferentConnection(u2, 1);
  hs1 = new HScrollbar(0, 25, width, 12, 16,width);
  u1.u=-1;
  v1.u=0.5;
  u2.u=.5;
  v2.u=-1;
  u1.computeOutput();
  u2.computeOutput();
  v1.computeOutput();
  v2.computeOutput();
  u1.compute();
  u1.u=-1;
  v1.compute();
  u2.compute();
  v2.compute();
  u1.computeOutput();
  u2.computeOutput();
  v1.computeOutput();
  v2.computeOutput();
  ctr=1;
}

void draw ()
{
  while (ctr<100)
  {
    u1.u=-1;
    ctr++;
  }
  noStroke();
  background( 255 );
  fill(0);
  text("u 1", 90, 70);
  text("v 1", 190, 70);
  text("u 2", 90, 170);
  text("v 2", 190, 170);
  text("adjust coupling strength", width/2-70, 10);
  text("0", 5, 40);
  text("1", width-10, 40);
  v1.afferent_weights[1]=(hs1.getPos())/(width);
  v2.afferent_weights[1]=(hs1.getPos())/(width);
  u1.computeOutput();
  u2.computeOutput();
  v1.computeOutput();
  v2.computeOutput();

 u2.compute();
 v2.compute();
 u1.compute();
 v1.compute();
 

  u1.drawConnections();
  v1.drawConnections();
  u2.drawConnections();
  v2.drawConnections();
  u1.drawActivation();
  v1.drawActivation();
  u2.drawActivation();
  v2.drawActivation();
  
   hs1.update();
  hs1.display();
}

class AmariNeuron
{
  //activation variable u
  float u;
  float dot_u;
  float output;
  //affernt neuron
  ArrayList afferent_neurons;
  float [] afferent_weights;
  //parameter section
  //timescale \tau
  float tau;
  //resting level
  float h;
  //slope \beta of sigmoid function
  float beta;
  //self excitation
  float c_uu;
  //noise strength
  float c_n;

  //afferent connection strength
  float c_afferent;
  //drawing parameters
  int pos_x, pos_y;
  int gray_value;
  float t,d_t,o_t;
  
  //constructor
  AmariNeuron(float u_0, float tau, float h, float beta, float c_uu, float c_n, int pos_x, int pos_y, int gray_value)
  {
    this.tau=tau;
    this.h=h;
    this.pos_x=pos_x;
    this.pos_y=pos_y;
    this.gray_value=gray_value;
    this.beta=beta;
    this.c_uu=c_uu;
    this.c_n=c_n;
    u=u_0;
    output=sigmoid(u, beta);
    afferent_neurons=new ArrayList(); 
    afferent_weights=new float[0];
    randomSeed(0);
  }
  void setAfferentConnection(AmariNeuron afferentNeuron, float c_afferent)
  {
    this.afferent_neurons.add(afferentNeuron);
    this.afferent_weights=append(this.afferent_weights, c_afferent);
  }
  void computeOutput()
  {
    output=sigmoid(u, beta);
  }
  void compute()
  {
    t=millis();
    //measure time since last call of loop
    d_t=t- o_t;
    o_t=t;
    if (d_t>tau)
    {
      d_t=tau;
    }
    if (d_t==0)
    {
      d_t=1;
    }
    float s=0;
    for (int i=0;i<afferent_neurons.size();i++)
    {
      AmariNeuron afferent_neuron= (AmariNeuron) afferent_neurons.get(i);
      c_afferent= afferent_weights[i];
      s+=c_afferent*afferent_neuron.output;
    }
    float n=random(-1, 1);
    dot_u=d_t/tau * (-u  + c_uu*output + s + c_n*n + h);
    u+=dot_u;
  }


  void drawConnections()
  { 
    for (int i=0;i<afferent_neurons.size();i++)
    {   
      AmariNeuron afferent_neuron= (AmariNeuron) afferent_neurons.get(i);
      //line(afferent_neuron.pos_x, afferent_neuron.pos_y, pos_x, pos_y);

      int distance_x=(pos_x-afferent_neuron.pos_x);
      int distance_y=(pos_y-afferent_neuron.pos_y);
      float distance=sqrt(pow(distance_x, 2)+pow(distance_y, 2));
      float direction=atan2(distance_y, distance_x);
      noFill();
      strokeWeight(2*abs(afferent_weights[i]));  
      stroke((1-afferent_neuron.output)*200); 
      float intermediate_point_x1=afferent_neuron.pos_x+distance/3*cos(direction) -  distance/3*sin(direction);
      float intermediate_point_x2=afferent_neuron.pos_x+2*distance/3*cos(direction) -  distance/3*sin(direction);
      float intermediate_point_y1=afferent_neuron.pos_y+distance/3*sin(direction) + distance/3*cos(direction);
      float intermediate_point_y2=afferent_neuron.pos_y+2*distance/3*sin(direction) +  distance/3*cos(direction);

      bezier(afferent_neuron.pos_x, afferent_neuron.pos_y, intermediate_point_x1, intermediate_point_y1, intermediate_point_x2, intermediate_point_y2, pos_x, pos_y);
      float bezier_x = bezierPoint(afferent_neuron.pos_x, intermediate_point_x1, intermediate_point_x2, pos_x, .8);
      float bezier_y = bezierPoint(afferent_neuron.pos_y, intermediate_point_y1, intermediate_point_y2, pos_y, .8);
      // ellipse(bezier_x,bezier_y,5,5);
      fill((1-afferent_neuron.output)*200);
      if (afferent_weights[i]<0)
      {
        direction=atan2( pos_y-bezier_y, pos_x-bezier_x);
        //pushMatrix();  
        float elipse_x=pos_x-(10+30*output)/2*cos(direction);
        float elipse_y=pos_y-(10+30*output)/2*sin(direction);

        //   translate(elipse_x, elipse_y);
        // rotate(direction);
        ellipse(elipse_x, elipse_y, 10, 10);
        //popMatrix();
      }
      else
      {
        direction=atan2( pos_y-bezier_y, pos_x-bezier_x);
        pushMatrix();  
        float elipse_x=pos_x-(10+30*output)/2*cos(direction);
        float elipse_y=pos_y-(10+30*output)/2*sin(direction);

        translate(elipse_x, elipse_y);
        rotate(direction+PI/4);
        rectMode(CENTER);
        rect(0, 0, 8, 8);
        popMatrix();
      }
    }
    if (c_uu!=0)
    {
      stroke((1-output)*200);      
      fill((1-output)*200);
      float bezier_x = bezierPoint(pos_x, pos_x-50, pos_x-50, pos_x, .8);
      float bezier_y = bezierPoint(pos_y, pos_y+50, pos_y-50, pos_y, .8);
      float direction=atan2( pos_y-bezier_y, pos_x-bezier_x);
      pushMatrix();  
      float elipse_x=pos_x-(10+30*output)/2*cos(direction);
      float elipse_y=pos_y-(10+30*output)/2*sin(direction);

      translate(elipse_x, elipse_y);
      rotate(direction+PI/4);
      rectMode(CENTER);
      rect(0, 0, 8, 8);
      popMatrix();
      noFill();

      bezier(pos_x, pos_y, pos_x- 50, pos_y + 50, pos_x-50, pos_y-50, pos_x, pos_y);
    }
  }
  void drawActivation()
  {
    noStroke();
    fill(gray_value);
    ellipse(pos_x, pos_y, 15+25*output, 15+25*output);
  }
  float sigmoid(float x, float beta)
  {
    return 1./(1+exp(-beta*x));
  }
}

class HScrollbar {
  int swidth, sheight;    // width and height of bar
  float xpos, ypos;       // x and y position of bar
  float spos, newspos;    // x position of slider
  float sposMin, sposMax; // max and min values of slider
  int loose;              // how loose/heavy
  boolean over;           // is the mouse over the slider?
  boolean locked;
  float ratio;

  HScrollbar (float xp, float yp, int sw, int sh, int l,int pos) {
    swidth = sw;
    sheight = sh;
    int widthtoheight = sw - sh;
    ratio = (float)sw / (float)widthtoheight;
    xpos = xp;
    ypos = yp-sheight/2;
    spos = pos;
    newspos = spos;
    sposMin = xpos;
    sposMax = xpos + swidth - sheight;
    loose = l;
  }

  void update() {
    if (overEvent()) {
      over = true;
    } else {
      over = false;
    }
    if (mousePressed && over) {
      locked = true;
    }
    if (!mousePressed) {
      locked = false;
    }
    if (locked) {
      newspos = constrain(mouseX-sheight/2, sposMin, sposMax);
    }
    if (abs(newspos - spos) > 1) {
      spos = spos + (newspos-spos)/loose;
    }
  }

  float constrain(float val, float minv, float maxv) {
    return min(max(val, minv), maxv);
  }

  boolean overEvent() {
    if (mouseX > xpos && mouseX < xpos+swidth &&
       mouseY > ypos && mouseY < ypos+sheight) {
      return true;
    } else {
      return false;
    }
  }

  void display() {
    noStroke();
    fill(204);
    rect(xpos, ypos, swidth*2, sheight);
    if (over || locked) {
      fill(0, 0, 0);
    } else {
      fill(102, 102, 102);
    }
    rect(spos, ypos, sheight, sheight);
  }

  float getPos() {
    // Convert spos to be values between
    // 0 and the total width of the scrollbar
    return spos * ratio;
  }
}


